#! /usr/bin/env python

import gtk, gtk.gdk, gtk.glade as glade, gnome.ui
from nxplot import *
from types import *
from copy import *
import gnumexp_conf
from string import zfill
import xml.dom.minidom as dom


class Color:
    def __init__(self, *args):
        """Color(name) or Color(red, green, blue)"""
	self.name = None
	self.red = self.green = self.blue = None
        if len(args) == 3:
            self.red = args[0]
            self.green = args[1]
            self.blue = args[2]
        elif len(args) == 1:
            color = args[0]
	    if color[0] == '#' and len(color) == 7:
		self.red   = int(color[1:3], 16) << 8
		self.green = int(color[3:5], 16) << 8
		self.blue  = int(color[5:7], 16) << 8
	    else:
		self.name = color
        else:
            raise ValueError("Invalid number of arguments")
	if self.name is not None:
	    c = gtk.gdk.color_parse(self.name)
	    self.red   = c.red
	    self.green = c.green
	    self.blue  = c.blue


    def __str__(self):
	return '#' + zfill(hex(int(self.red >> 8))[2:], 2) + \
	       zfill(hex(int(self.green >> 8))[2:], 2)     + \
	       zfill(hex(int(self.blue >> 8))[2:], 2)

    def distance(self, other):

	"""Calculate a number indicating how much a color differs from
	another. Range: 0-1.0"""

	return (abs(self.red   - other.red) + 
		abs(self.green - other.green) + 
		abs(self.blue  - other.blue)) / (3*65536.0)


def _linemode_to_string(mode):
    if   mode == Linemode.SOLID:		return 'solid'
    elif mode == Linemode.DOTTED:		return 'dotted'
    elif mode == Linemode.DOTDASHED:		return 'dotdashed'
    elif mode == Linemode.SHORTDASHED:		return 'shortdashed'
    elif mode == Linemode.LONGDASHED:		return 'longdashed'
    elif mode == Linemode.DOTDOTDASHED:		return 'dotdotdashed'
    elif mode == Linemode.DOTDOTDOTDASHED:	return 'dotdotdotdashed'
    elif mode == Linemode.DISCONNECTED:		return 'disconnected'
    else:					return 'invalid!'

def _linemode_from_string(mode):
    if   mode == 'solid':		return Linemode.SOLID
    elif mode == 'dotted':		return Linemode.DOTTED
    elif mode == 'dotdashed':		return Linemode.DOTDASHED
    elif mode == 'shortdashed':		return Linemode.SHORTDASHED
    elif mode == 'longdashed':		return Linemode.LONGDASHED
    elif mode == 'dotdotdashed':	return Linemode.DOTDOTDASHED
    elif mode == 'dotdotdotdashed':	return Linemode.DOTDOTDOTDASHED
    elif mode == 'disconnected':	return Linemode.DISCONNECTED
    else:				return Linemode.SOLID


class LineAttr:
    def __init__(self, color=Color(0,0,0), thickness=1.0,
		 dash_style=Linemode.SOLID):
	assert isinstance(color, Color)
	assert type(dash_style) is IntType
	assert Linemode_valid(dash_style)
	self.dash_style = dash_style
        self.color      = color
        self.thickness  = thickness

    def apply_to(self, pl, width_scale):
	assert isinstance(pl, Nxplot)
	# color
	if self.color.name:
	    pl.pencolorname(self.color.name)
	else:
	    pl.pencolor(self.color.red, self.color.green, self.color.blue)
	# dash style
	pl.linemode(self.dash_style)
	# width
	pl.linewidth(self.thickness*width_scale)

    def get_linewidth(self, width_scale):
	return self.thickness*width_scale

    def to_xml(self, doc):
	assert isinstance(doc, dom.Document)
	elem = doc.createElement("lineattr")
	elem.setAttribute("color", str(self.color))
	elem.setAttribute("mode",  _linemode_to_string(self.dash_style))
	elem.setAttribute("width", str(self.thickness))
	return elem
    
    def from_xml(self, elem):
	assert isinstance(elem, dom.Element)
	assert elem.tagName == 'lineattr'
	self.color      = Color(elem.getAttribute("color"))
	self.dash_style = _linemode_from_string(elem.getAttribute("mode"))
	self.thickness  = float(elem.getAttribute("width"))

    def __str__(self):
	return "LineAttr(thickness=%f, dash_style=%s, color='%s')" % \
	       (self.thickness, _linemode_to_string(self.dash_style),
		str(self.color))

    def distance(self, other):
	"""Calculate a number indicating how much a lineattr differs from another"""
	return abs(self.dash_style - other.dash_style) + \
	       abs(self.thickness - other.thickness) + \
	       10*Color.distance(self.color, other.color)
	

class LineAttrPreview(gtk.Image):
    def __init__(self, attr, border=1, width=-1, height=-1):
	assert isinstance(attr, LineAttr)
	gtk.Image.__init__(self)
	self.attr = attr
	self.draw_border = border
	if width == -1:
	    self.pl = NxplotArtft(self)
	    self.pl.connect('update', self.update)
	else:
	    pl = NxplotArtft(width, height)
	    self.update(pl)
	    pixbuf = pl.get_pixbuf()
	    self.set_from_pixbuf(pixbuf)

    def update(self, pl=None):
	if pl is None: pl = self.pl
	pl.open()
	pl.space(0, 0, 2, 1)
	pl.erase()

	pl.linemode(self.attr.dash_style)
	if self.attr.color.name is None:
	    pl.pencolor(self.attr.color.red, self.attr.color.green,
			self.attr.color.blue)
	else:
	    pl.pencolorname(self.attr.color.name)

	pl.linewidth(0.0125*self.attr.thickness)
	pl.line(.1, .5, 1.9, .5)
	pl.endpath()

	if self.draw_border:
	    pl.linewidth(0.05)
	    pl.pencolor(0, 0, 0)
	    pl.linemode(Linemode.SOLID)
	    d=.01
	    x1, y1 = d, d
	    x2, y2 = 2 - d, 1 - d
	    pl.box(x1, y1, x2, y2)

	pl.close()


	

class LineAttrDialog:
    def __init__(self, attr):
	self.attr = attr; del attr
	self.xml = glade.XML(gnumexp_conf.GLADEDIR + "/lineattr.glade2")
	self.dialog = self.xml.get_widget("LineAttr")

	self.spin = self.xml.get_widget("line_width")
	self.xml.get_widget("width label").set_mnemonic_widget(self.spin)

	self.spin.set_value(self.attr.thickness)

	self.color_picker = self.xml.get_widget("line_color")
	self.xml.get_widget("color label").set_mnemonic_widget(self.color_picker)
	self.color_picker.set_i16(self.attr.color.red,
				  self.attr.color.green,
				  self.attr.color.blue, 0)

	opmenu = self.xml.get_widget("line_style")
	self.xml.get_widget("line style label").set_mnemonic_widget(opmenu)
	menu = gtk.Menu()
	i = -1
	for n in Linemode_values()[:-1]:
	    i += 1
	    if n == self.attr.dash_style:
		history = i
	    item = gtk.MenuItem()
	    menu.append(item)
	    attr = LineAttr(thickness=10.0, dash_style=n, color=Color(0,0,0))
	    prev = LineAttrPreview(attr, width=80, height=15)
	    item.add(prev)
	    item.show_all()
	    item.set_data("dash-style", n)
	opmenu.set_menu(menu)
	opmenu.set_history(history)
	opmenu.connect("changed", self.__set_dash_style)
	self.preview = LineAttrPreview(self.attr, border=0)
	self.preview.set_size_request(100, 50)
	self.preview.show()
	preview_frame = self.xml.get_widget("preview_frame")
	preview_frame.add(self.preview)

	self.xml.signal_autoconnect({
	    'lineattr_window_preview_update': self.preview_update
	    })
	#self.preview_update()



    def __set_dash_style(self, opmenu):
	item = opmenu.get_menu().get_active()
	style = item.get_data("dash-style")
	self.attr.dash_style = style
	self.preview_update()

    def preview_update(self, *args):
	self.attr.color = Color(self.color_picker.get_property("red"),
				self.color_picker.get_property("green"),
				self.color_picker.get_property("blue"))
	self.attr.thickness = self.spin.get_value()
	if self.preview.window is not None:
	    self.preview.update()


class LineAttrButton(gtk.Button):
    def __init__(self, attr):
	gtk.Button.__init__(self)
	self.attr = attr

	alignment = gtk.Alignment(0.5, 0.5, 0.75, 0.25)
	alignment.set_border_width(1)
	self.add(alignment)
	alignment.show()

	frame = gtk.Frame()
	frame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
	alignment.add(frame)
	frame.show()

	self.preview = LineAttrPreview(attr, border=0)
	self.preview.show()
	self.preview.set_size_request(64, 16)
	frame.add(self.preview)

	self.dialog = None
	self.connect("clicked", LineAttrButton.__clicked)

    def __dialog_destroyed(self, dialog):
	self.dialog = None

    def __dialog_response(self, dialog, response):
 	if response == gtk.RESPONSE_ACCEPT:
	    self.attr = self.dialog.attr
	    self.preview.attr = self.attr
	    if self.preview.window is not None:
		self.preview.update()
	dialog.destroy()

    def __clicked(self):
	if self.dialog:
	    self.dialog.dialog.present()
	    return
	self.dialog = LineAttrDialog(deepcopy(self.attr))
	self.dialog.dialog.set_transient_for(self.get_toplevel())
	self.dialog.dialog.connect("destroy", self.__dialog_destroyed)
	self.dialog.dialog.connect("response", self.__dialog_response)
	self.dialog.dialog.show()
	
    def set_attr(self, attr):
	self.attr = attr
	self.preview.attr = attr
	#if self.preview.window is not None:
	#    self.preview.update()
	

def _test_dialog():
    attr = LineAttr(thickness=10.0, dash_style=Linemode.LONGDASHED, color=Color(0,0,65535))
    win = gtk.Window()
    bt = LineAttrButton(attr)
    win.add(bt)
    bt.show()
    win.connect('destroy', gtk.mainquit)
    win.show()
    gtk.mainloop()


def _test_xml():
    doc  = dom.Document()
    attr = LineAttr(thickness=10.0, dash_style=Linemode.LONGDASHED, color=Color(0,0,65535))
    elem = attr.to_xml(doc)
    print elem
    print elem.toxml()
    attr1 = LineAttr()
    attr1.from_xml(elem)
    print str(attr)
    print str(attr1)
    doc.appendChild(elem)
    doc.writexml(file("/tmp/lineattr.xml", "w"))


if __name__ == '__main__':
    _test_dialog()
    #_test_xml()

