import warnings
import itertools
from misc import *
from kernel import *
from plotobject_registry import *
from dialogs.registry import *
import dialogs.util
import xml.dom.minidom as dom
import types, traceback
import ORBit, CORBA
import xyconsts
from gettext import gettext as _
def N_(message): return message


class Function(PlotObject):

    def __init__(self, values, plotarea):
        self.expression = None
        self.variable = None
	self.tree = None
	self.bbox = None
	self.samples = None
	self.mathml = '<math><mtext></mtext></math>'

        PlotObject.__init__(self)
	self.caption = "%e"

	# keep this last!
	if type(values) is DictionaryType:
	    self.set_values(values)
	elif isinstance(values, dom.Element):
	    self.from_xml(values, plotarea)
	else:
	    raise TypeError("expected dictionary or DOM Element")

        self.set_state(self.STATE_NORMAL)
        

    def get_values(self):
	values = PlotObject.get_values(self)
	values['contents'] = { 'expression': self.expression,
			       'independent-variable': 	self.variable,
			       }
	values['type'] = 'Function'
	return values

    def set_values(self, values):
	PlotObject.set_values(self, values)
	values = values['contents']
	self.expression = values['expression']
	self.variable   = values['independent-variable']
    	self.update_function()
	self.__update_mathml()

    def __update_mathml(self):
	try:
	    el = kernel.parseString(self.expression)
	    expression_mathml = kernel.elementToMathML(el)
	    kernel.elementUnref(el)
	except Exception, ex:
	    expression_mathml = '<merror>error converting expression to MathML: %s</merror>' % str(ex)
	expression_mathml = expression_mathml.decode("utf-8")
	print "MathML: ", `expression_mathml`
	variable_mathml = '<mi>' + self.variable + '</mi>'
	self.mathml = self._label_to_mathml(self.caption,
					    {'%e': expression_mathml,
					     '%v': variable_mathml})
    
    def __del__(self):
	if self.tree:
	    kernel.elementUnref(self.tree)
	self.tree = None

    def update_samples(self):
	if self.canvas is None: return
	try:
	    self.sample()
	except CORBA.UserException, ex:
	    self.samples = None
	    err = str(ex.__class__) + ": " + ex.detail
	    dialog = gtk.MessageDialog(
		flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
		type           = gtk.MESSAGE_ERROR,
		buttons        = gtk.BUTTONS_OK,
		message_format = err)
	    dialog.set_property("has-separator", gtk.FALSE)
	    self.view.set_transient_for(dialog)
	    dialog.run()
	    dialog.destroy()
	    return
	except Exception, ex:
	    self.samples = None
	    dialog = gtk.MessageDialog(
		flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
		type           = gtk.MESSAGE_ERROR,
		buttons        = gtk.BUTTONS_OK,
		message_format = _("Unknown error while evaluating expression:\n") +
		                 str(ex.__class__))
	    self.view.set_transient_for(dialog)
	    dialog.run()
	    dialog.destroy()
	    raise
	self.queue_draw()
	self.update_control_points()


    def set_range(self, x1, y1, x2, y2):
	PlotObject.set_range(self, x1, y1, x2, y2)
	self.update_samples()

    def update_function(self):
	self.update_samples()

    def sample(self):
        return self._adaptive_sample()

    def _linear_sample(self):
	if self.expression is None: return
	nsamples = 200

	yvalues = numexp_kernel_sample(self.expression, self.variable,
				       self.range.x1, self.range.x2, nsamples)
	
	xvalues = numexp_kernel_sample(self.variable, self.variable,
				       self.range.x1, self.range.x2, nsamples)
	
	# Combine X and Y values and store the result
	self.samples = zip(xvalues, yvalues)

    def _adaptive_sample(self):
	if self.expression is None: return

        values = numexp_kernel_adaptive_sample(self.expression, self.variable,
                                               self.range.x1, self.range.x2,
                                               self.range.y1, self.range.y2,
                                               3000)
        self.samples = list(itertools.izip(
            itertools.islice(values, 0, len(values), 2),
            itertools.islice(values, 1, len(values), 2)))


    def get_control_points(self):
	if not self.samples: return ()
	pl = self.get_plotter()
	if not isinstance(pl, NxplotArtft): return ()
	rv = []
	min_distance = 10
	last_wx = None
	last_wy = None
	for x, y in self.samples:
            if y is None: # discontinuity?
                continue
	    wx, wy = pl.space2window(self.range2bbox_x(x),
				     self.range2bbox_y(y))
	    if last_wx is None:
		rv.append((wx, wy))
		last_wx, last_wy = wx, wy
	    else:
                try:
                    d = math.hypot(wx - last_wx, wy - last_wy)
                except OverflowError:
                    continue
		if d > min_distance:
		    rv.append((wx, wy))
		    last_wx, last_wy = wx, wy
	return rv


    def real_draw(self, pl):
	if not self.samples: return
	for lineattr in (self.outer_lineattr, self.inner_lineattr):
            if lineattr is None:
                continue
	    pl.endpath()
	    lineattr.apply_to(pl, xyconsts.WIDTH_SCALE)
	    x1 = None
	    y1 = None
	    for (x, y) in self.samples:
                if y is None: # discontinuity?
                    x1 = None
                    x2 = None
                    continue
		if x1 is None or y1 is None:
		    x1 = self.range2bbox_x(x)
		    y1 = self.range2bbox_y(y)
		else:
		    x2 = self.range2bbox_x(x)
		    y2 = self.range2bbox_y(y)
		    pl.line(x1, y1, x2, y2)
		    x1, y1 = x2, y2
	    pl.endpath()


    def draw_legend_sample(self, pl, really_draw):
	thickness = self.lineattr.thickness*xyconsts.WIDTH_SCALE
	y = thickness/2
	width = thickness*10
	if not really_draw:
	    return width, thickness
	else:
	    self.lineattr.apply_to(pl, xyconsts.WIDTH_SCALE)
	    pl.line(0, 0, width, 0)
	    pl.endpath()
    
    def to_xml(self, doc):
	assert isinstance(doc, dom.Document)
	elem = PlotObject.to_xml(self, doc)
	elem.setAttribute("type", "Function")
	expr = doc.createElement("expression")
	expr.appendChild(doc.createTextNode(self.expression))
	elem.appendChild(expr)
	var = doc.createElement("variable")
	var.setAttribute("name", self.variable)
	elem.appendChild(var)
	return elem

    def from_xml(self, elem, plotarea):
	PlotObject.from_xml(self, elem, plotarea)
	assert elem.getAttribute("type") == 'Function'
	expr = elem.getElementsByTagName('expression')[0]
	self.expression = ""
	for node in expr.childNodes:
	    if node.nodeType == node.TEXT_NODE:
		self.expression += node.data
	self.expression = self.expression.strip().encode()
	var = elem.getElementsByTagName('variable')[0]
	self.variable = var.getAttribute("name").encode()
	self.__update_mathml()

    # override legend label drawing function to enable mathml
    def draw_legend_label(self, pl, attributes, really_draw, data=None):
	return self._draw_legend_label_mathml(self.mathml, pl, attributes,
					      really_draw, data)

    def get_y_range(self):
	if self.samples is None: return None
	yvalues = [p[1] for p in self.samples]
	return min(yvalues), max(yvalues)
	

class FunctionContents(PlotObjectContents):
    def __init__(self):
	PlotObjectContents.__init__(self, "func contents function")
	self.name = "Function"

	self.expression_wid = dialogs.util.ExpressionEntry(kernel)
        self.expression_wid.show()
	self.xml.get_widget("expression container").add(self.expression_wid)
	self.xml.get_widget("expression label").set_mnemonic_widget(self.expression_wid)

	self.independent_var_wid = dialogs.util.VariableEntry(self.expression_wid)
        self.independent_var_wid.show()
	self.xml.get_widget("variable container").add(self.independent_var_wid)
	self.xml.get_widget("variable label").set_mnemonic_widget(self.independent_var_wid)

    def get_values(self):
	rv = {'expression': 		self.expression_wid.get_text(),
	      'independent-variable': 	self.independent_var_wid.get_value(),
	      }
	return rv

    def set_values(self, values):
	self.expression_wid.set_text(values['expression'])
	self.independent_var_wid.set_value(values['independent-variable'])


plotobjectcontents_list.append(FunctionContents)
type_name_to_class_mapping[N_('Function')] = Function
__all__ = ['Function']
print "'Function' plot object has been registered"

